// Lazy 7 / One - Test Sketch v2 - nodeMCU / ESP8266 support (experimental)
// https://www.thingiverse.com/thing:4363658
// https://www.instructables.com/id/Lazy-7-One/
// 06/2020 - Daniel Cikic
// Serial is set to 74880

// uncomment #define nodeMCU to compile this for nodeMCU v1.0 (ESP-12E) instead of Arduino Pro Mini/Nano
// #define nodeMCU

// uncomment below to use 6 digits (HH:MM:SS), do NOT manually change LED_DIGITS for this!
// #define use6D

#ifdef nodeMCU
  #define FASTLED_ESP8266_RAW_PIN_ORDER       // this means we'll be using the raw esp8266 pin order -> GPIO_12, which is d6 on nodeMCU
  #define LED_PIN 12                          // led data in connected to GPIO_12 (d6/nodeMCU)
#else
  #define FASTLED_ALLOW_INTERRUPTS 0
  #define LED_PIN 6                           // led data in connected to d6 (arduino)
#endif

#define LED_PWR_LIMIT 500                     // 500mA - Power limit in mA (voltage is set in setup() to 5v)
#ifdef use6D
  #define LED_DIGITS 6                        // digit count (4 or 6, defined by use6D)
#else
  #define LED_DIGITS 4
#endif
#define LED_PER_DIGIT_STRIP 56                // each digit is using 56 leds (7 segments / 8 leds each)
#define SPACING_LEDS 2                        // 2 unused leds between digits/center dots
#define LED_PER_CENTER_MODULE 12              // 12 leds inside center module (6 upper dot/6 lower dot)
// Using all of the above we can calculate total led count:
#define LED_COUNT LED_DIGITS * LED_PER_DIGIT_STRIP + ( LED_DIGITS / 2 ) * ( SPACING_LEDS * 2 ) + ( LED_DIGITS / 3 ) * LED_PER_CENTER_MODULE + ( LED_DIGITS / 3 ) * ( SPACING_LEDS * 4 )

#include <FastLED.h>

byte brightness = 100;
byte saturation = 255;
byte startColor = 0;

CRGB leds[LED_COUNT];

#ifdef use6D                                  // because of the way the first digit is built (first _and_ last led inside its segments) we need different "specs" for 4/6 digits
  const uint16_t segGroups[7][4] PROGMEM {    // Each segment has 1-x led(s). So lets assign them in a way we get the first digit (right one when seen from the front) completely.
    {   4,   7,  16,  19 },                   // top, a
    {   0,   3,  20,  23 },                   // top right, b
    { 368, 371, 372, 375 },                   // bottom right, c
    { 364, 367, 376, 379 },                   // bottom, d
    { 360, 363, 380, 383 },                   // bottom left, e
    {   8,  11,  12,  15 },                   // top left, f
    {  24,  27, 384, 387 },                   // center, g
  };
#else
  const uint16_t segGroups[7][4] PROGMEM = {  // Each segment has 1-x led(s). So lets assign them in a way we get the first digit (right one when seen from the front) completely.
    {   4,   7,  16,  19 },                   // top, a
    {   0,   3,  20,  23 },                   // top right, b
    { 232, 235, 236, 239 },                   // bottom right, c
    { 228, 231, 240, 243 },                   // bottom, d
    { 224, 227, 244, 247 },                   // bottom left, e
    {   8,  11,  12,  15 },                   // top left, f
    {  24,  27, 248, 251 },                   // center, g
  };
#endif
// All other digits/segments can be calculated based on this and the definitions on top of the sketch
// Note: The first number always has to be the lower one as they're subtracted later on... (fix by using abs()? ^^)

const byte digits[10][7] PROGMEM = {        // Lets define 10 numbers (0-9) with 7 segments each, 1 = segment is on, 0 = segment is off
  {   1,   1,   1,   1,   1,   1,   0 },    // 0 -> Show segments a - f, don't show g (center one)
  {   0,   1,   1,   0,   0,   0,   0 },    // 1 -> Show segments b + c (top and bottom right), nothing else
  {   1,   1,   0,   1,   1,   0,   1 },    // 2 -> and so on...
  {   1,   1,   1,   1,   0,   0,   1 },    // 3
  {   0,   1,   1,   0,   0,   1,   1 },    // 4
  {   1,   0,   1,   1,   0,   1,   1 },    // 5
  {   1,   0,   1,   1,   1,   1,   1 },    // 6
  {   1,   1,   1,   0,   0,   0,   0 },    // 7
  {   1,   1,   1,   1,   1,   1,   1 },    // 8
  {   1,   1,   1,   1,   0,   1,   1 }     // 9
};

void setup() {
  Serial.begin(74880); Serial.println();    
  Serial.println(F("Lazy 7 / One - Test Sketch v2 starting up..."));
  Serial.print(F("Configured for: ")); Serial.print(LED_COUNT); Serial.println(F(" leds"));
  Serial.print(F("Power limited to (mA): ")); Serial.print(LED_PWR_LIMIT); Serial.println(F(" mA")); Serial.println();
  FastLED.addLeds<WS2812B, LED_PIN, GRB>(leds, LED_COUNT).setCorrection(TypicalSMD5050).setTemperature(DirectSunlight).setDither(1);
  FastLED.setMaxPowerInVoltsAndMilliamps(5, LED_PWR_LIMIT);
  FastLED.clear();
  FastLED.show();
}

void loop() {
  delay(500);

  // simply lighting first to last led up...
  for ( int i = 0; i < LED_COUNT; i++ ) {
    leds[i].setHSV(0, saturation, brightness);
    FastLED.show();
    delay(50);
  }

  delay(500);
  FastLED.clear(); FastLED.show();
  delay(500);

  // count from 0-9 on position 0-3 (right to left)
  for ( byte pos = 0; pos < LED_DIGITS; pos++ ) {
    for ( int i = 0; i < 10; i++ ) {
      showDigit(i, startColor, pos);
      FastLED.show();
      delay(150);
      FastLED.clear();
      startColor += 16;
    }
    delay(150);
  }
  
  delay(500);
  FastLED.show();
  delay(500);

  // count from 0-99 using the right two (seen from front) digits
  byte varSpeed = 220;
  for ( byte counter = 0; counter < 100; counter++ ) {
    showDigit(counter % 10, startColor, 0);
    showDigit(counter / 10, startColor, 1);
    FastLED.show();
    delay(varSpeed);
    FastLED.clear();
    startColor++;
    if ( varSpeed > 25 ) varSpeed -= 5;
  }
  
  delay(500);
  FastLED.show();
  delay(500);

  varSpeed = 220;

  // count from 0-99 using the left two (seen from front) digits (center when using 6 digits)
  for ( byte counter = 0; counter < 100; counter++ ) {
    showDigit(counter % 10, startColor, 2);
    showDigit(counter / 10, startColor, 3);
    FastLED.show();
    delay(varSpeed);
    FastLED.clear();
    startColor++;
    if ( varSpeed > 25 ) varSpeed -= 5;
  }

#ifdef use6D
  delay(500);
  FastLED.show();
  delay(500);

  varSpeed = 220;

  // count from 0-99 using the left two (seen from front) digits (only when using 6 digits)
  for ( byte counter = 0; counter < 100; counter++ ) {
    showDigit(counter % 10, startColor, 4);
    showDigit(counter / 10, startColor, 5);
    FastLED.show();
    delay(varSpeed);
    FastLED.clear();
    startColor++;
    if ( varSpeed > 25 ) varSpeed -= 5;
  }
#endif

  delay(500);
  FastLED.show();
  delay(500);
  startColor++;
}

void showSegment(byte segment, byte color, byte segDisplay) {
  // This shows the segments from top of the sketch on a given position (segDisplay).
  // pos 0 is the most right one (seen from the front) where data in is connected
  int startLED = pgm_read_word_near(&segGroups[segment][0]);
  int endLED = pgm_read_word_near(&segGroups[segment][1]);
  int offsetLED = 0;
  if ( segDisplay > 0 ) offsetLED = segDisplay * LED_PER_DIGIT_STRIP / 2 + segDisplay * SPACING_LEDS;               // if position/display is greater 0 we add half the leds of a digit
  if ( segDisplay >= 2 ) offsetLED += segDisplay / 2 * LED_PER_CENTER_MODULE / 2 + segDisplay / 2 * SPACING_LEDS;   // if position/display is greater 1 we have to add offsets for the dots
  for (int i = startLED; i <= endLED; i++) {                                                  // light up group 1 (1st and 2nd value inside segGroups array)
    if ( segment == 0 || segment == 1 || segment == 5 ) leds[i + offsetLED].setHSV(color, saturation, brightness);  // upper 3 segments
    if ( segment == 2 || segment == 3 || segment == 4 ) leds[i - offsetLED].setHSV(color, saturation, brightness);  // lower 3 segments
    if ( segment == 6 ) leds[i + offsetLED].setHSV(color, saturation, brightness);                                  // upper part of center segment
  }
  startLED = pgm_read_word_near(&segGroups[segment][2]);
  endLED = pgm_read_word_near(&segGroups[segment][3]);
  for (int i = startLED; i <= endLED; i++) {                                                  // light up group 2 (3rd and 4th value inside segment array)
    if ( segment == 0 || segment == 1 || segment == 5 ) leds[i + offsetLED].setHSV(color, saturation, brightness);  // upper 3 segments
    if ( segment == 2 || segment == 3 || segment == 4 ) leds[i - offsetLED].setHSV(color, saturation, brightness);  // lower 3 segments
    if ( segment == 6 ) leds[i - offsetLED].setHSV(color, saturation, brightness);                                  // lower part of center segment
  }
}

void showDigit(byte digit, byte color, byte pos) {
  // This draws numbers using the according segments as defined on top of the sketch (0 - 9)
  for (byte i = 0; i < 7; i++) {
    if ( pgm_read_byte_near(&digits[digit][i]) != 0) showSegment(i, color, pos);
  }
}

